"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;
function _default(babel) {
  var t = babel.types;
  return {
    name: "babel-plugin-react-slot",
    visitor: {
      JSXAttribute: function JSXAttribute(path) {
        var attrName = path.node.name.name;
        var attrValue = path.node.value.value;
        if (attrName !== "slot") {
          return;
        }

        // 1. add slotted element to parent as props
        // <Box><div slot="foo">bar</div></Box>
        // to
        // <Box __slot_foo={<div>bar</div>}></Box>
        var element = path.findParent(t.isJSXElement);
        var parentElement = element.findParent(t.isJSXElement);
        var clonedElement = t.cloneNode(element.node);
        clonedElement.openingElement.attributes = clonedElement.openingElement.attributes.filter(function (e) {
          return e.name.name !== "slot";
        });
        var slotAttribute = t.JSXAttribute(t.JSXIdentifier("__slot_" + attrValue), t.JSXExpressionContainer(clonedElement));
        if (parentElement.node.openingElement.attributes) {
          parentElement.node.openingElement.attributes.push(slotAttribute);
        }
        element.remove();
      },
      JSXElement: function JSXElement(path) {
        var _slotAttrNameNode$val;
        var openingElement = path.node.openingElement;
        var openingElementName = openingElement.name.name;
        if ((openingElementName === null || openingElementName === void 0 ? void 0 : openingElementName.toLowerCase()) !== "slot") {
          return;
        }

        // SLOT setup
        var slotAttrNameNode = openingElement.attributes.find(function (attr) {
          return attr.name.name === "name";
        });
        var slotValue = slotAttrNameNode === null || slotAttrNameNode === void 0 || (_slotAttrNameNode$val = slotAttrNameNode.value) === null || _slotAttrNameNode$val === void 0 ? void 0 : _slotAttrNameNode$val.value;
        var slotElmenetChildren = path.node.children;

        // COMPONENT function setup
        var component = path.getFunctionParent();
        var firstParam = component.node.params[0];
        if (component.isArrowFunctionExpression()) {
          component.arrowFunctionToExpression();
        }

        // UNNAMED Element SLOT
        // {p.children} or {props.children} or {children} case
        var slotReplacementName = "children";
        if (slotAttrNameNode) {
          // NAMED Element SLOT
          // {x, y, z, ..., __slot_name} case
          slotReplacementName = "__slot_" + slotValue;
        }

        // 1. Set param to function expression
        var _setParamToComponent = setParamToComponent(t, component, firstParam, slotReplacementName),
          firstParamId = _setParamToComponent.firstParamId,
          firstParamIsId = _setParamToComponent.firstParamIsId;

        // 2. Replace <slot> with {props.children} or {children} or {props.__slot_x} or {__slot_x}
        transformSlotTag(t, path, firstParamId, firstParamIsId, slotElmenetChildren, slotReplacementName);
      }
    }
  };
}
function setParamToComponent(t, component, firstParam, childrenName) {
  var firstParamId = t.isAssignmentPattern(firstParam) ? firstParam.left : firstParam;
  var firstParamIsId = true;
  var idChildren = t.identifier(childrenName);
  if (component.node.params.length === 0) {
    firstParamId = t.identifier("props");
    component.node.params = [t.identifier("props")];
  } else if (t.isObjectPattern(component.node.params[0])) {
    component.node.params[0].properties.push(t.objectProperty(t.cloneNode(idChildren), t.cloneNode(idChildren), false, true));
    firstParamIsId = false;
  }
  return {
    firstParamId: firstParamId,
    firstParamIsId: firstParamIsId
  };
}
function transformSlotTag(t, path, firstParamId, firstParamIsId, slotElmenetChildren, propName) {
  var getPropIdentifier = function getPropIdentifier() {
    return t.Identifier(propName);
  };
  if (firstParamIsId) {
    if (slotElmenetChildren.length > 0) {
      //     {props.__slot_x ? props.__slot_x : <>Default content</>}
      path.replaceWith(t.JSXExpressionContainer(t.conditionalExpression(t.MemberExpression(t.cloneNode(firstParamId), getPropIdentifier()), t.MemberExpression(t.cloneNode(firstParamId), getPropIdentifier()), t.JSXFragment(t.JSXOpeningFragment(), t.JSXClosingFragment(), slotElmenetChildren))));
    } else {
      //     {props.__slot_x}
      path.replaceWith(t.JSXExpressionContainer(t.MemberExpression(t.cloneNode(firstParamId), getPropIdentifier())));
    }
  } else {
    if (slotElmenetChildren.length > 0) {
      //     {__slot_x ? __slot_x : <>Default content</>}
      path.replaceWith(t.JSXExpressionContainer(t.conditionalExpression(getPropIdentifier(), getPropIdentifier(), t.JSXFragment(t.JSXOpeningFragment(), t.JSXClosingFragment(), slotElmenetChildren))));
    } else {
      //     {__slot_x}
      path.replaceWith(t.JSXExpressionContainer(getPropIdentifier()));
    }
  }
}
